@@ -5,8 +5,9 @@ SPDX-License-Identifier: GPL-2.0-or-later
 */
 #pragma once
 
+#include <QDBusConnection>
+#include <QDBusObjectPath>
 #include <QObject>
-#include <QtDBus>
 
 class FakeLogindSession;
 
