@@ -49,7 +49,6 @@ SPDX-License-Identifier: GPL-2.0-or-later
 #include <QQuickItem>
 #include <QQuickView>
 
-#include <private/qtx11extras_p.h>
 // Wayland
 #include <wayland-client.h>
 #include <wayland-ksld-client-protocol.h>
@@ -155,7 +154,7 @@ UnlockApp::UnlockApp(int &argc, char **argv)
     m_authenticators = new PamAuthenticators(std::move(interactive), std::move(noninteractive), this);
     initialize();
 
-    if (QX11Info::isPlatformX11()) {
+    if (KWindowSystem::isPlatformX11()) {
         installNativeEventFilter(new FocusOutEventFilter);
     }
 }
@@ -339,7 +338,7 @@ PlasmaQuick::QuickViewSharedEngine *UnlockApp::createViewForScreen(QScreen *scre
     view->engine()->setNetworkAccessManagerFactory(new NoAccessNetworkAccessManagerFactory);
 
     if (!m_testing) {
-        if (QX11Info::isPlatformX11()) {
+        if (KWindowSystem::isPlatformX11()) {
             view->setFlags(Qt::X11BypassWindowManagerHint);
         } else {
             view->setFlags(Qt::FramelessWindowHint);
@@ -595,16 +594,17 @@ bool UnlockApp::eventFilter(QObject *obj, QEvent *event)
                 break;
             }
         }
-        if (view && view->winId() && QX11Info::isPlatformX11()) {
+        if (view && view->winId() && KWindowSystem::isPlatformX11()) {
             // showing greeter view window, set property
-            static Atom tag = XInternAtom(QX11Info::display(), "_KDE_SCREEN_LOCKER", False);
-            XChangeProperty(QX11Info::display(), view->winId(), tag, tag, 32, PropModeReplace, nullptr, 0);
+            auto x11App = qGuiApp->nativeInterface<QNativeInterface::QX11Application>();
+            static Atom tag = XInternAtom(x11App->display(), "_KDE_SCREEN_LOCKER", False);
+            XChangeProperty(x11App->display(), view->winId(), tag, tag, 32, PropModeReplace, nullptr, 0);
         }
         // no further processing
         return false;
     }
 
-    if (event->type() == QEvent::MouseButtonPress && QX11Info::isPlatformX11()) {
+    if (event->type() == QEvent::MouseButtonPress && KWindowSystem::isPlatformX11()) {
         if (getActiveScreen()) {
             getActiveScreen()->requestActivate();
         }
@@ -612,12 +612,10 @@ bool UnlockApp::eventFilter(QObject *obj, QEvent *event)
     }
 
     if (event->type() == QEvent::KeyPress) { // react if saver is visible
-        shareEvent(event, qobject_cast<PlasmaQuick::QuickViewSharedEngine *>(obj));
         return false; // we don't care
     } else if (event->type() == QEvent::KeyRelease) { // conditionally reshow the saver
         QKeyEvent *ke = static_cast<QKeyEvent *>(event);
         if (ke->key() != Qt::Key_Escape) {
-            shareEvent(event, qobject_cast<PlasmaQuick::QuickViewSharedEngine *>(obj));
             return false; // irrelevant
         } else if (!m_testing) { // don't turn screen off in testing mode.
             auto dpms = new KScreen::Dpms(this);
@@ -638,34 +636,6 @@ bool UnlockApp::eventFilter(QObject *obj, QEvent *event)
     return false;
 }
 
-/*
- * This function forwards an event from one greeter window to all others
- * It's used to have the keyboard operate on all greeter windows (on every screen)
- * at once so that the user gets visual feedback on the screen he's looking at -
- * even if the focus is actually on a powered off screen.
- */
-
-void UnlockApp::shareEvent(QEvent *e, PlasmaQuick::QuickViewSharedEngine *from)
-{
-    // from can be NULL any time (because the parameter is passed as qobject_cast)
-    // m_views.contains(from) is atm. supposed to be true but required if any further
-    // QQuickView are added (which are not part of m_views)
-    // this makes "from" an optimization (nullptr check aversion)
-    if (from && m_views.contains(from)) {
-        // NOTICE any recursion in the event sharing will prevent authentication on multiscreen setups!
-        // Any change in regarded event processing shall be tested thoroughly!
-        removeEventFilter(this); // prevent recursion!
-        const bool accepted = e->isAccepted(); // store state
-        for (PlasmaQuick::QuickViewSharedEngine *view : std::as_const(m_views)) {
-            if (view != from) {
-                QCoreApplication::sendEvent(view, e);
-                e->setAccepted(accepted);
-            }
-        }
-        installEventFilter(this);
-    }
-}
-
 void UnlockApp::setGraceTime(int milliseconds)
 {
     m_graceTime = milliseconds;
