@@ -16,7 +16,7 @@ class KScreenLockerData : public KCModuleData
     Q_OBJECT
 
 public:
-    explicit KScreenLockerData(QObject *parent, const QVariantList &args);
+    explicit KScreenLockerData(QObject *parent);
 
     bool isDefaults() const override;
 
