@@ -99,7 +99,7 @@ KScreenSaverSettings::~KScreenSaverSettings()
 {
 }
 
-QVector<WallpaperInfo> KScreenSaverSettings::availableWallpaperPlugins() const
+QList<WallpaperInfo> KScreenSaverSettings::availableWallpaperPlugins() const
 {
     return m_availableWallpaperPlugins;
 }
@@ -115,3 +115,5 @@ void KScreenSaverSettings::setShortcut(const QKeySequence &sequence)
 }
 
 #include "kscreensaversettings.moc"
+
+#include "moc_kscreensaversettings.cpp"
